@@ -1,32 +1,34 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import threading
-import inspect
-from inspect import  signature
+import six
 import collections
-import os
-from enum import Enum, unique
-import sys
+import datetime
+import inspect
+import json
 import linecache
 import math
+import os
+import platform
+import re
 import shlex
 import struct
-import platform
 import subprocess
-import numpy as np
-import re
-from pydoc import locate
-import datetime
+import sys
+import threading
 import time
-import types
 import traceback
-import json
+import types
+from enum import Enum, unique
+from inspect import signature
+from pydoc import locate
 from typing import List, Set, Tuple, Dict
 
+import numpy as np
+
 __all__ = ['get_session','get_trident_dir','get_signature','epsilon','set_epsilon','floatx','set_floatx','if_else','camel2snake','snake2camel','to_onehot','to_list','addindent','format_time', 'get_time_suffix', 'get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime', 'next_prime', 'prev_prime', 'nearest_prime','PrintException','unpack_singleton','enforce_singleton','OrderedDict','get_python_function_arguments','map_function_arguments','ClassfierType','PaddingMode', 'DataRole',
 
-           'ExpectDataType', 'GetImageMode','split_path', 'make_dir_if_need','sanitize_path', 'ShortcutMode', 'DataSpec','get_argument_maps','get_args_spec']
+           'ExpectDataType', 'GetImageMode','split_path', 'make_dir_if_need','sanitize_path', 'ShortcutMode', 'DataSpec','get_argument_maps','get_args_spec','get_gpu_memory_map']
 
 def sanitize_path(path):
     if isinstance(path,str) :
@@ -166,7 +168,8 @@ def get_signature(fn,skip_default=False):
     parameters = list(inspect.signature(fn).parameters.values())
     for k in parameters:
         if skip_default==False or (skip_default==True and k.default ==inspect._empty):
-            signature[k.name]= k.default if k.default !=inspect._empty else None
+            if k.name not in ['args','kwargs']:
+                signature[k.name]= k.default if k.default !=inspect._empty else None
     return signature
 
 
@@ -289,7 +292,9 @@ def unpack_singleton(x):
     # Returns
         The same iterable or the first element.
     """
-    if isinstance(x,(tuple,list)) and len(x) == 1:
+    if 'tensor' in x.__class__.__name__.lower()or isinstance(x,np.ndarray):
+        return x
+    elif isinstance(x,(tuple,list)) and len(x) == 1:
         return x[0]
     return x
 
@@ -301,7 +306,9 @@ def enforce_singleton(x):
     # Returns
         The same iterable or the first element.
     """
-    if hasattr(x,'__len__'):
+    if 'tensor' in x.__class__.__name__.lower() or isinstance(x,np.ndarray):
+        return x
+    elif hasattr(x,'__len__'):
         return x[0]
     return x
 
@@ -810,3 +817,90 @@ def update_signature(fn:callable,args:list):
 
 
 
+
+def get_gpu_memory_map():
+    """Get the current gpu usage.
+
+    Returns
+    -------
+    usage: dict
+        Keys are device ids as integers.
+        Values are memory usage as integers in MB.
+    """
+    pathes = [p for p in os.environ['path'].split(';') if 'NVIDIA' in p and 'Corporation' in p]
+    nv_path='C:/Program Files/NVIDIA Corporation/'
+    sp = subprocess.Popen(['{0}/NVSMI/nvidia-smi'.format(nv_path), '-q'],encoding='utf-8-sig', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+    out_str = sp.communicate()
+    out_list = out_str[0].split('\n')
+
+    # Convert lines into a dictionary
+
+    gpu_memory_map = {}
+    for item in out_list:
+        try:
+            key, val = item.split(':')
+            key, val = key.strip(), val.strip()
+            gpu_memory_map[key] = val
+        except:
+            pass
+    return gpu_memory_map
+
+
+class _tensor_op(collections.MutableMapping):
+    """Tensor manuplation utilities
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.__dict__.update(*args, **kwargs)
+
+    def __setitem__(self, key, value):
+        self.__dict__[key] = value
+
+    def __getitem__(self, key):
+        return self.__dict__[key]
+
+    def __delitem__(self, key):
+        del self.__dict__[key]
+
+    # noinspection PyUnusedLocal,PyUnusedLocal
+    def __getattr__(self, key):
+        return None
+
+    def __iter__(self):
+        return iter(self.__dict__)
+
+    def __len__(self):
+        return len(self.__dict__)
+
+    def __str__(self):
+        return str(self.__dict__)
+
+    def __repr__(self):
+        return self.__dict__.__repr__()
+
+    def __add__(self, other):
+        """Overloads `+` operator.
+        It does NOT overwrite the existing item.
+        """
+
+        res = _tensor_op(self.__dict__)
+        for k, v in six.iteritems(other):
+            if k not in res.__dict__ or res.__dict__[k] is None:
+                res.__dict__[k] = v
+        return res
+
+
+    def __mul__(self, other):
+        r"""Overloads `*` operator.
+        It overwrites the existing item.
+
+
+        ```
+        """
+        res = _tensor_op(self.__dict__)
+        for k, v in six.iteritems(other):
+            res.__dict__[k] = v
+        return res
+
+
