@@ -1,37 +1,33 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
-import collections
+import copy
 import inspect
+import logging
+import operator
+import os
+import random
+import re
+import sys
+import uuid
+import warnings
+from collections import defaultdict
+from collections import deque
+from copy import copy, deepcopy
+from functools import partial, wraps, update_wrapper
+from itertools import islice
 
 import matplotlib
 import matplotlib.pyplot as plt
-from copy import copy, deepcopy
-from collections import deque
+import numpy as np
 import six
-import random
-import logging
-import uuid
-import os
-import copy
-import sys
-import re
-import warnings
-from collections import defaultdict
-from functools import partial,wraps,update_wrapper
-from itertools import islice
-import operator
 import torch
 import torch.nn as nn
 import torch.onnx
 import torchvision
 
-import numpy as np
-from .common import to_list,addindent,camel2snake,snake2camel,unpack_singleton,enforce_singleton,OrderedDict,get_signature
+from .common import to_list, addindent, camel2snake, snake2camel, unpack_singleton, enforce_singleton, OrderedDict, \
+    get_signature
 from .pytorch_ops import *
 
-__all__ = ['get_uid','to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call']
+__all__ = ['get_uid','to_numpy','to_tensor','is_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call']
 
 version=torch.__version__
 sys.stderr.write('Pytorch version:{0}.\n'.format(version))
@@ -58,7 +54,8 @@ from typing import List, IO, Union, Tuple, Type, Callable
 
 
 
-
+def is_tensor(x):
+    return isinstance(x,torch.Tensor)
 
 def to_numpy(x) -> np.ndarray:
 
@@ -127,44 +124,6 @@ def get_uid(prefix=''):
     _UID_PREFIXES[prefix] += 1
     return _UID_PREFIXES[prefix]
 
-# class InputSpec(torch.IntTensor):
-#   """Specifies the ndim, dtype and shape of every input to a layer.
-#   Every layer should expose (if appropriate) an `input_spec` attribute:
-#   a list of instances of InputSpec (one per input tensor).
-#   A None entry in a shape is compatible with any dimension,
-#   a None shape is compatible with any shape.
-#   Arguments:
-#       dtype: Expected DataType of the input.
-#       shape: Shape tuple, expected shape of the input
-#           (may include None for unchecked axes).
-#       ndim: Integer, expected rank of the input.
-#       max_ndim: Integer, maximum rank of the input.
-#       min_ndim: Integer, minimum rank of the input.
-#       axes: Dictionary mapping integer axes to
-#           a specific dimension value.
-#   """
-#
-#   def __init__(self, input_tensor_shape):
-#
-#     @property
-#     def ndim(self):
-#         return len(self.size())
-#
-#     @property
-#     def shape(self):
-#         return tuple(self.tolist())
-#
-#   def __repr__(self):
-#     spec = [('dtype=' + str(self.dtype)) if self.dtype else '',
-#             ('shape=' + str(self.shape)) if self.shape else '',
-#             ('ndim=' + str(self.ndim)) if self.ndim else '',
-#             ('max_ndim=' + str(self.max_ndim)) if self.max_ndim else '',
-#             ('min_ndim=' + str(self.min_ndim)) if self.min_ndim else '',
-#             ('axes=' + str(self.axes)) if self.axes else '']
-#     return 'InputSpec(%s)' % ', '.join(x for x in spec if x)
-
-
-
 
 class Layer(nn.Module):
     """
@@ -301,7 +260,6 @@ class Layer(nn.Module):
     def get_weights(self):
         return [p.numpy() for p in list(self.parameters())]
 
-
     def set_weights(self, weights):
         params = self.weights
         if not params:
@@ -316,7 +274,6 @@ class Layer(nn.Module):
         for p, w in weight_value_tuples:
             p.data = w.data
 
-
     @property
     def trainable(self):
         if len(self.weights)==0:
@@ -402,6 +359,8 @@ class Layer(nn.Module):
                 value = value[0].int()
             else:
                 value=to_tensor(to_numpy(list(value)))
+        elif isinstance(value, int):
+            value=to_tensor(to_numpy([value]))
         elif isinstance(value, torch.Size):
             value=to_tensor(to_numpy(list(value))).int()
         elif isinstance(value, torch.Tensor):
@@ -443,6 +402,8 @@ class Layer(nn.Module):
                 value = value[0].int()
             else:
                 value = to_tensor(to_numpy(list(value)))
+        elif isinstance(value, int):
+            value=to_tensor(to_numpy([value]))
         elif isinstance(value, torch.Size):
             value = to_tensor(to_numpy(list(value))).int()
         elif isinstance(value, torch.Tensor):
@@ -481,6 +442,7 @@ class Layer(nn.Module):
     def copy(self):
         return copy.deepcopy(self)
 
+
     def save_onnx(self,file_path=''):
         input_shape=self.input_shape.copy()
         input_shape.insert(0,1)
@@ -907,6 +869,7 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
 
             m_key =module.name if hasattr(module,'name') else camel2snake(module.__class__.__name__) + '_' + str(get_uid(module.__class__.__name__))
             summary[m_key] = OrderedDict()
+            summary[m_key]["keep_output"] = module.keep_output
             summary[m_key]["input_shape"] = list(input[0].size())
             summary[m_key]["input_shape"][0] = batch_size
             if isinstance(output, (list, tuple)):
@@ -987,9 +950,10 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
     macc=0
     for layer in summary:
         # input_shape, output_shape, trainable, nb_params
+        is_keep = '★' if summary[layer]["keep_output"] else ''
         line_new = "{0:<40s} {1:<20s}  {2:<20s} {3:<8s}  {4:<8}  {5:<12}".format(
             layer,
-            str(summary[layer]["output_shape"]),
+            is_keep+str(summary[layer]["output_shape"]),
             str(summary[layer]["weight"] if 'weight' in summary[layer] else ''),
             str(summary[layer]["bias"] if 'bias' in summary[layer] else ''),
             summary[layer]["nb_params"],
@@ -1005,11 +969,12 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
         print(line_new)
 
     # assume 4 bytes/number (float on cuda).
-    total_input_size = abs(np.prod(input_size) * batch_size * 4. / (1024 ** 2.))
-    total_output_size = abs(2. * total_output * 4. / (1024 ** 2.))  # x2 for gradients
-    total_params_size = abs(total_params * 4. / (1024 ** 2.))
+    total_input_size = np.abs(np.prod(np.array(list(input_size))) * batch_size * 4. / (1024 ** 2.))
+    total_output_size = np.abs(2. * total_output * 4. / (1024 ** 2.))  # x2 for gradients
+    total_params_size = np.abs(total_params * 4. / (1024 ** 2.))
     total_size = total_params_size + total_output_size + total_input_size
 
+
     print("================================================================")
     print("Total params: {0:,}".format(total_params))
     print("Trainable params: {0:,}".format(trainable_params))
