@@ -1,16 +1,18 @@
 import os
 import sys
-from typing import Tuple, Optional, Union
 from collections import Sized, Iterable
+from functools import partial
+from typing import Tuple,List, Optional, Union
+
+import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 from torch.autograd import Variable
-from functools import partial
-import numpy as np
 from .common import *
+
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-__all__ = ['is_nan','is_abnormal_number','reshape','argmax','gram_matrix','log_sum_exp','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
+__all__ = ['is_nan','is_abnormal_number','reshape','argmax','gram_matrix','log_sum_exp','reduce_mean','reduce_max','reduce_min','reduce_sum','sqrt','square','abs','exp','log','pow','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
 
 
 
@@ -28,21 +30,79 @@ def is_nan(x):
     else:
         raise NotImplementedError
 
-def is_abnormal_number(x):
+def is_inf(x):
     if isinstance(x,torch.Tensor):
-        return torch.isnan(x).any() or torch.isinf(x).any()or torch.isinf(-x).any()
+        return torch.isinf(x).any()
     elif isinstance(x,nn.Module):
         for para in x.parameters():
-            if torch.isnan(para).any() or torch.isinf(para).any()or torch.isinf(-para).any():
+            if torch.isinf(para).any():
                 return True
         return False
     elif isinstance(x, np.ndarray):
-        return np.isnan(x).any() or np.isinf(x).any() or np.isneginf(x).any()
+        return np.isinf(x).any()
     else:
         raise NotImplementedError
 
+def is_abnormal_number(x):
+    return is_nan(x) or is_inf(x)or is_inf(-x)
 
 
+def clip(x:torch.Tensor,min_value=-np.inf,max_value=np.inf):
+    return x.clamp(min,max)
+
+def sqrt(x:torch.Tensor):
+    return x.sqrt()
+
+def square(x:torch.Tensor):
+    return x**2
+
+
+def abs(x:torch.Tensor):
+    return x.abs()
+
+def pow(x:torch.Tensor,y):
+    return x.pow(y)
+
+def log(x:torch.Tensor):
+    return x.log()
+
+def exp(x:torch.Tensor):
+    return x.exp()
+
+
+def reduce_mean(x:torch.Tensor,axis=None,keepdims=False,**kwargs):
+    axis=kwargs.get('dim',axis)
+    keepdims = kwargs.get('keepdim', keepdims)
+    if isinstance(axis,int):
+        return x.mean(dim=axis,keepdim=keepdims)
+    elif isinstance(axis, list)  :
+        axis=sorted(axis).reverse()
+        for a in axis:
+            x=x.mean(dim=a,keepdim=keepdims)
+        return x
+
+def reduce_sum(x:torch.Tensor,axis=None,keepdims=False,**kwargs):
+    axis = kwargs.get('dim', axis)
+    keepdims = kwargs.get('keepdim', keepdims)
+    return x.sum(dim=axis,keepdim=keepdims)
+
+def reduce_max(x:torch.Tensor,axis=None,keepdims=False,**kwargs):
+    axis = kwargs.get('dim', axis)
+    keepdims = kwargs.get('keepdim', keepdims)
+    arr, idx = x.max(dim=axis,keepdim=keepdims)
+    return arr
+
+def reduce_min(x:torch.Tensor,axis=None,keepdims=False,**kwargs):
+    axis = kwargs.get('dim', axis)
+    keepdims = kwargs.get('keepdim', keepdims)
+    arr, idx = x.min(dim=axis,keepdim=keepdims)
+    return arr
+
+mean=reduce_mean
+sum=reduce_sum
+max=reduce_max
+min=reduce_min
+
 
 def reshape(x,shape=None)-> torch.Tensor:
     if shape is None:
@@ -61,16 +121,16 @@ def gram_matrix(input):
     # (c,d)=dimensions of a f. map (N=c*d)
 
     features = input.view(a * b, c * d)  # resise F_XL into \hat F_XL
-
+    features=features-features.mean(-1)
     G = torch.mm(features, features.t())  # compute the gram product
 
     # we 'normalize' the values of the gram matrix
     # by dividing by the number of element in each feature maps.
-    return G.div(a * b * c * d)
+    return G#.div(a * b * c * d)
 
 
 
-def log_sum_exp(x,axis=1)-> torch.Tensor:
+def log_sum_exp(x:torch.Tensor,axis=1)-> torch.Tensor:
 
     x_max = x.data.max()
     return torch.log(torch.sum(torch.exp(x-x_max), dim=axis, keepdim=True)) + x_max
@@ -88,18 +148,10 @@ def argmax(x:torch.Tensor,axis=1)-> torch.Tensor:
         _, idx = x.max()
     return idx
 
-def reduce_mean(t:torch.Tensor,axis=1,keepdim=False):
-    return t.mean(dim=axis,keepdim=keepdim)
-def reduce_sum(t:torch.Tensor,axis=1,keepdim=False):
-    return t.sum(dim=axis,keepdim=keepdim)
 
-def reduce_max(t:torch.Tensor,axis=1,keepdim=False):
-    arr, idx = t.max(dim=axis,keepdim=keepdim)
-    return arr
 
-def reduce_min(t:torch.Tensor,axis=1,keepdim=False):
-    arr, idx = t.min(dim=axis,keepdim=keepdim)
-    return arr
+
+
 
 def expand_dims(t:torch.Tensor,axis=0):
     return t.unsqueeze(axis)
