@@ -1,15 +1,18 @@
-import warnings
 import math
-import numpy as np
 import os
 import sys
-from ..callbacks import CallbackBase
+import warnings
+
+import numpy as np
+
 from ..backend.common import *
 from ..backend.load_backend import *
 from ..backend.pillow_backend import image2array
-from ..misc.ipython_utils import is_in_ipython, is_in_colab
+from ..callbacks import CallbackBase
 from ..data.mask_common import label2color
+from ..misc.ipython_utils import is_in_ipython, is_in_colab
 from ..misc.visualization_utils import *
+
 if is_in_ipython() or is_in_colab():
     from IPython import display
 
@@ -238,8 +241,7 @@ class PlotLossMetricsCallback(VisualizationCallbackBase):
         self.training_items = training_context['training_items']
 
     def on_overall_batch_end(self, training_context):
-        if self.batch_inteval > 0 and (
-                self.training_items.value_list[0].training_context['current_batch'] + 1) % self.batch_inteval == 0:
+        if (self.batch_inteval > 0 and (self.training_items.value_list[0].training_context['current_batch'] + 1) % self.batch_inteval == 0) or (self.epoch_inteval > 0 and self.training_items.value_list[0].training_context['current_batch'] +1==self.training_items.value_list[0].training_context['total_batch']  and (self.training_items.value_list[0].training_context['current_epoch'] + 1) % self.epoch_inteval == 0):
             if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
                 display.clear_output(wait=True)
                 self.counter = 0
@@ -255,6 +257,7 @@ class PlotLossMetricsCallback(VisualizationCallbackBase):
                               max_iteration=None, save_path=os.path.join(self.save_path, self.name_prefix),
                               imshow=self.imshow)
 
+
             # if self.tile_image_unit == 'epoch' and (epoch + 1) % self.tile_image_frequency == 0:  #     epoch_loss_history = [trainitem.epoch_loss_history for k, trainitem in self.training_items.items()]  #     epoch_metric_history = [trainitem.epoch_metric_history for k, trainitem in self.training_items.items()]  #  #     loss_metric_curve(epoch_loss_history, epoch_metric_history, legend=self.training_names.value_list,  #                       calculate_base='epoch', max_iteration=self.num_epochs,  #                       save_path=os.path.join(self.tile_image_save_path, 'loss_metric_curve.png'),  #                       imshow=True)
 
 
