@@ -1,22 +1,25 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import six
+
+import math
 import os
 import random
-import time
 import re
+import time
+from itertools import repeat
+
+import cv2
 import numpy as np
-import math
-from  scipy import  ndimage
+import six
+from scipy import ndimage
+from skimage import color
 from skimage import exposure
 from skimage import morphology
-from skimage import color
-from skimage import  transform,exposure
+from skimage import transform, exposure
 from skimage.filters import *
+
 from ..backend.common import *
-from itertools import repeat
-import cv2
 
 __all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','list_pictures','normalize','unnormalize','channel_reverse','blur','random_blur','random_crop','resize','rescale','downsample_then_upsample','add_noise','gray_scale','to_rgb','auto_level','random_invert_color',
            'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contrast','random_adjust_contrast','clahe','erosion_then_dilation','dilation_then_erosion','image_erosion','image_dilation','adaptive_binarization','random_transform',
@@ -801,7 +804,9 @@ def random_transform(rotation_range= 12, zoom_range= 0.05, shift_range=0.05,rand
 def to_low_resolution(scale=2):
     def img_op(image: np.ndarray):
         rnd=random.randint(0,10)
-        if rnd%3==0:
+        if rnd==0:
+            image = rescale(1 / scale)(image)
+        elif rnd%3==0:
             image=downsample_then_upsample(scale+2)(image)
             image=add_noise(0.02)(image)
             image=rescale(1/scale)(image)
