@@ -1,33 +1,36 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+
+import glob
+import glob
+import gzip
+import hashlib
 import json
-import threading
 import os
+import shutil
 import subprocess
 import sys
-import glob
-import tqdm
-from  sys import stderr
-import requests
-import warnings
-import numpy as np
-from tqdm import tqdm
-import gzip
 import tarfile
+import threading
+import urllib.request
+import warnings
 import zipfile
-import shutil
-import hashlib
-import glob
-import six
-from scipy.io import loadmat
-from urllib3.exceptions import NewConnectionError
+from sys import stderr
 
-from ..backend.common import OrderedDict,PrintException
+import numpy as np
+import requests
+import six
+import tqdm
+from scipy.io import loadmat
 from six.moves.urllib.error import HTTPError
 from six.moves.urllib.error import URLError
 from six.moves.urllib.request import urlopen
-import urllib.request
+from tqdm import tqdm
+from urllib3.exceptions import NewConnectionError
+
+from ..backend.common import OrderedDict, PrintException
+
 try:
     from urllib.request import urlretrieve
 except ImportError:
@@ -167,7 +170,7 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None):
         except Exception as e:
             print('***Cannot download data, so the data provider cannot initialized.\n', flush=True)
             print( '***Please check your internet or download  files from following url in another computer, \n and then put them into {0}\n {1} '.format(dirname, 'https://drive.google.com/open?id={0}'.format(file_id)), flush=True)
-            Print(e)
+            print(e)
             return False
 
 
@@ -337,6 +340,11 @@ def extract_archive(file_path, target_folder=None, archive_format='auto'):
   return False
 
 
+def pickle_it(file_path,dict):
+    import pickle as pickle
+    with open(file_path, 'wb') as handle:
+        pickle.dump(dict, handle, protocol=2)
+
 def unpickle(file):
     import _pickle as pickle
     with open(file, 'rb') as fo:
