@@ -2,17 +2,18 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-from ..backend.common import  epsilon,get_function,camel2snake
-import numpy as np
 import cntk as C
-from cntk.ops.functions import ModelFormat, CloneMethod, Function, BlockFunction, load_model, register_native_user_function, native_user_function
-from cntk.internal import sanitize_input, sanitize_shape, sanitize_axis, sanitize_dynamic_axes, sanitize_axis_list, sanitize_multi_axis_reduction_list, typemap, sanitize_pooling_args, sanitize_convolution_args, sanitize_permutation, sanitize_dtype_cntk
-
-
-from cntk.ops.functions import Function, BlockFunction
-
+import numpy as np
 import six
+from cntk.internal import sanitize_input, sanitize_shape, sanitize_axis, sanitize_dynamic_axes, sanitize_axis_list, \
+    sanitize_multi_axis_reduction_list, typemap, sanitize_pooling_args, sanitize_convolution_args, \
+    sanitize_permutation, \
+    sanitize_dtype_cntk
+from cntk.ops.functions import Function, BlockFunction
+from cntk.ops.functions import ModelFormat, CloneMethod, Function, BlockFunction, load_model, \
+    register_native_user_function, native_user_function
 
+from ..backend.common import epsilon, get_function, camel2snake
 
 __all__ = ['Identity','Sigmoid','Tanh','Relu','Relu6','LeakyRelu','LeakyRelu6','SmoothRelu','PRelu','Swish','Elu','HardSigmoid','HardSwish','Selu','LecunTanh','SoftSign','SoftPlus','HardTanh','Logit','LogLog','Mish','Softmax','identity','sigmoid','tanh','relu','relu6','leaky_relu','leaky_relu6','smooth_relu','p_relu','swish','elu','hard_sigmoid','hard_swish','selu','lecun_tanh','soft_sign','soft_plus','hard_tanh','logit','log_log','mish','softmax','get_activation']
 
