@@ -644,7 +644,7 @@ class ShortCut2d(Layer):
         super(ShortCut2d, self).__init__(name=name)
         self.activation = get_activation(activation)
         self.has_identity = False
-        self.mode = mode if isinstance(mode, str) else mode.value
+        self.mode = mode if isinstance(mode, str) else mode
         self.output_idx=output_idx
         self.skip_tensor=None
         for i in range(len(args)):
@@ -652,22 +652,27 @@ class ShortCut2d(Layer):
             if isinstance(arg, (Layer,torch.Tensor, list, dict)):
                 if isinstance(arg, list):
                     arg = Sequential(*arg)
-                elif isinstance(arg, dict) and len(args) == 1:
+                elif isinstance(arg, (dict,OrderedDict)) and len(args) == 1:
                     for k, v in arg.items():
                         if isinstance(v, Identity):
                             self.has_identity = True
                             self.add_module('Identity', v)
                         else:
                             self.add_module(k, v)
-                elif isinstance(arg, dict) and len(args) > 1:
+                elif isinstance(arg,  (dict,OrderedDict)) and len(args) > 1:
                     raise ValueError('more than one dict argument is not support.')
                 elif isinstance(arg, torch.Tensor):
                     self.skip_tensor=arg
                 elif isinstance(arg, Identity):
                     self.has_identity = True
                     self.add_module('Identity', arg)
+                elif isinstance(arg, nn.Module):
+                    if len(arg.name)>0 and arg.name!=arg.defaultname:
+                        self.add_module(arg.name, arg)
+                    else:
+                        self.add_module('branch{0}'.format(i + 1), arg)
                 else:
-                    self.add_module('branch{0}'.format(i + 1), arg)
+                    raise ValueError('{0} is not support.'.format(arg.__class__.__name))
         if len(self._modules) == 1 and self.has_identity == False and self.output_idx is None:
             self.has_identity = True
             self.add_module('Identity', Identity())
@@ -676,28 +681,36 @@ class ShortCut2d(Layer):
 
     def forward(self, *x):
         x = enforce_singleton(x)
-        result = []
+
+        current=None
         if self.has_identity == True:
-            result.append(x)
+            current=x
         for k, v in self._modules.items():
             if not isinstance(v, Identity):
-                result.append(v(x))
+                if current is None:
+                    current=v(x)
+                else:
+                    if (not hasattr(self, 'mode') or self.mode == 'add'):
+                        current=current+v(x)
+                    elif self.mode == 'dot':
+                        current =current* v(x)
+                    elif self.mode == 'concate':
+                        current=torch.cat([current,v(x)], dim=1)
+                    else:
+                        raise ValueError('Not valid shortcut mode')
+        x=current
         if hasattr(self,'skip_tensor') and self.skip_tensor is not None:
-            result.append(self.skip_tensor)
-
-        if (not hasattr(self, 'mode') or self.mode == 'add'):
-            if result[0].shape!=result[1].shape:
-                raise RuntimeError('Shape is not compatable between  {0} and {1}'.format(result[0].shape,result[1].shape))
-            result = Add()(result)
-        elif self.mode == 'dot':
-            result = Dot()(result)
-        elif self.mode == 'concate':
-            result = torch.cat(result, dim=1)
-        else:
-            raise ValueError('Not valid shortcut mode')
+            if (not hasattr(self, 'mode') or self.mode == 'add'):
+                x = x+self.skip_tensor
+            elif self.mode == 'dot':
+                x =x* self.skip_tensor
+            elif self.mode == 'concate':
+                x = torch.cat([x, self.skip_tensor], dim=1)
+            else:
+                raise ValueError('Not valid shortcut mode')
         if self.activation is not None:
-            result = self.activation(result)
-        return result
+            x = self.activation(x)
+        return x
 
 
 class ConcateBlock(Layer):
@@ -769,10 +782,8 @@ class SqueezeExcite(Layer):
 
     def build(self, input_shape):
         if self._built == False :
-            self.squeeze = Conv2d((1, 1), self.se_filters, strides=1, auto_pad=False, activation=None,
-                                  use_bias=self.use_bias, name=self.name + '_squeeze')
-            self.excite = Conv2d((1, 1), self.num_filters, strides=1, auto_pad=False, activation=None,
-                                 use_bias=self.use_bias, name=self.name + '_excite')
+            self.squeeze = Conv2d((1, 1), self.se_filters, strides=1, auto_pad=False, activation=None,use_bias=self.use_bias, name=self.name + '_squeeze')
+            self.excite = Conv2d((1, 1), self.num_filters, strides=1, auto_pad=False, activation=None, use_bias=self.use_bias, name=self.name + '_excite')
             self.to(self.device)
             self._built = True
 
