@@ -1,30 +1,31 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import math
+
 import inspect
+import math
+from collections import OrderedDict
 from functools import partial, wraps, update_wrapper
 from itertools import islice
-import torch
+from itertools import repeat
 
+import numpy as np
+import torch
 import torch.nn as nn
+import torch.nn.functional as F  # import torch functions
 import torch.utils.hooks as hooks
+import torchvision
+from torch._jit_internal import List
+from torch._six import container_abcs
 from torch.nn import Module
 from torch.nn import init
 from torch.nn.parameter import Parameter
-import torch.nn.functional as F  # import torch functions
-from torch._six import container_abcs
-from torch._jit_internal import List
-from itertools import repeat
 
-from collections import OrderedDict
+from .pytorch_activations import get_activation
+from .pytorch_normalizations import *
 from ..backend.common import *
 from ..backend.pytorch_backend import *
 from ..backend.pytorch_ops import meshgrid
-from .pytorch_activations import get_activation
-import torchvision
-from .pytorch_normalizations import *
-import numpy as np
 
 __all__ = ['Dense', 'Flatten', 'Concatenate', 'Concate','SoftMax','Add', 'Subtract', 'Dot', 'Conv1d', 'Conv2d', 'Conv3d',
            'TransConv1d', 'TransConv2d', 'TransConv3d', 'SeparableConv1d', 'SeparableConv2d', 'SeparableConv3d',
@@ -85,7 +86,7 @@ class Dense(Layer):
         torch.Size([128, 30])
     """
 
-    def __init__(self, output_shape, use_bias=True, activation=None, name='', **kwargs):
+    def __init__(self, output_shape, use_bias=True, activation=None,keep_output=False, name='', **kwargs):
         super(Dense, self).__init__()
 
         if isinstance(output_shape, int):
@@ -97,6 +98,7 @@ class Dense(Layer):
         else:
             raise ValueError('output_shape should be integer, list of integer or tuple of integer...')
         self.name = name
+        self.keep_output=keep_output
         self.weight = None
         self.bias = None
         self.use_bias = use_bias
@@ -113,6 +115,7 @@ class Dense(Layer):
             self.to(self.device)
             self._built = True
 
+
     def forward(self, *x):
         x = enforce_singleton(x)
         x = F.linear(x, self.weight, self.bias)
@@ -142,21 +145,11 @@ class Flatten(Layer):
 
 
 class Concate(Layer):
-    r"""Flatten layer to flatten a tensor after convolution."""
+    r"""Concate layer to splice  tensors ."""
 
     def __init__(self, axis=1):
         super(Concate, self).__init__()
         self.axis = axis
-
-    def build(self, input_shape):
-        if self._built == False:
-
-            self.output_shape = input_shape[0]
-            for inp in input_shape[1:]:
-                self.output_shape[0] = self.output_shape[0] + inp[0]
-
-            self._built = True
-
     def forward(self, *x) -> torch.Tensor:
         if not isinstance(x, list) or len(x) < 2:
             raise ValueError('A `Concatenate` layer should be called on a list of at least 2 inputs')
