@@ -1,20 +1,22 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import math
+
 import collections
+import inspect
 import itertools
+import math
+from itertools import repeat
+
 import tensorflow as tf
-import tensorflow.keras.backend as K
-from tensorflow_core.python.keras.utils import conv_utils
+from  tensorflow.keras import backend as K
+from tensorflow.python.client import device_lib
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.keras.engine.input_spec import InputSpec
+from tensorflow.python.keras.utils import conv_utils
 from tensorflow.python.ops import nn_ops
-from tensorflow.python.client import device_lib
-from .tensorflow_activations import get_activation
-from itertools import repeat
-import inspect
 
+from .tensorflow_activations import get_activation
 from ..backend.common import *
 from ..backend.tensorflow_backend import *
 
@@ -98,39 +100,49 @@ class MaxPool1d(tf.keras.layers.MaxPool1D):
         return s.format(**self.__dict__)
 
 
-class MaxPool2d(tf.keras.layers.MaxPool2D):
-    def __init__(self,kernel_size = (2, 2), strides=None, auto_pad=True,padding_mode='replicate', **kwargs ):
-        kernel_size=_pair(kernel_size)
-        strides=_pair(strides or kernel_size)
-        super(MaxPool2d, self).__init__( kernel_size,strides,
-                 padding='same' if auto_pad else 'valid', data_format='channels_last', **kwargs)
-
-    @property
-    def kernel_size(self):
-        return super().pool_size
-
-    @kernel_size.setter
-    def kernel_size(self,value):
-        self.pool_size=_pair(value)
-    def __repr__(self):
-        return get_layer_repr(self)
-    def extra_repr(self):
-        s = 'kernel_size={pool_size}, strides={strides}'
-        s += ',auto_pad={0}'.format(self.padding == 'same')
-        return s.format(**self.__dict__)
+class MaxPool2d(Layer):
+    def __init__(self,kernel_size = (2, 2), strides=None, auto_pad=True,padding_mode='replicate', name=None,**kwargs ):
+        super(MaxPool2d, self).__init__(name=name)
+        self.kernel_size=_pair(kernel_size)
+        self.strides=_pair(strides or kernel_size)
+        self.auto_pad = auto_pad
+        self.padding = 'VALID'
+        if self.auto_pad == True:
+            self.padding = 'SAME'
+    def build(self, input_shape):
+        if self._built == False:
+            self._built = True
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        return tf.nn.max_pool2d(x, self.kernel_size, self.strides, self.padding, 'NHWC', self._name)
 
 
-class GlobalAvgPool2d(tf.keras.layers.Layer):
-    def __init__(self,name=''):
+
+class GlobalAvgPool2d(Layer):
+    def __init__(self, keepdim=False, name='avg_pool'):
         super(GlobalAvgPool2d, self).__init__(name=name)
+        self.keepdim = keepdim
 
-    def call(self, inputs, **kwargs):
-            if K.int_shape(inputs)[-1]==1:
-                return K.squeeze(K.mean(inputs, axis=[1, 2]))
-            return K.mean(inputs, axis=[1, 2])
-    def __repr__(self):
-        return get_layer_repr(self)
 
+    def build(self, input_shape):
+        if self._built == False:
+            if self.keepdim == True:
+                output_shape = input_shape.clone()
+                output_shape[0] = 1
+                output_shape[1] = 1
+                self.output_shape = output_shape
+            else:
+                self.output_shape = input_shape[-1]
+            self._built = True
+
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        if self.keepdim == True:
+            x = tf.reduce_mean(x,[1,2],keepdims=True)
+
+        else:
+            x = tf.reduce_mean(x,[1,2],keepdims=False)
+        return x
 
 
 def get_pooling(fn_name):
