@@ -1,9 +1,15 @@
-import tkinter as tk
-from .ipython_utils import is_in_ipython,is_in_colab
+from .ipython_utils import is_in_ipython, is_in_colab
+
 if is_in_ipython():
     from IPython import display
-import matplotlib
-matplotlib.use('Qt5Agg' if not is_in_ipython() and not is_in_colab() else 'TkAgg' if not is_in_colab() else 'NbAgg' )
+
+if not is_in_colab:
+    import matplotlib
+    matplotlib.use('Qt5Agg' if not is_in_ipython() and not is_in_colab() else 'NbAgg' )
+else:
+    import matplotlib
+
+
 
 import matplotlib.pyplot as plt
 from matplotlib.collections import PolyCollection
@@ -263,12 +269,7 @@ def plot_confusion_matrix(cm, class_names, figsize=(16, 16), normalize=False, ti
     """Render the confusion matrix and return matplotlib's figure with it.
     Normalization can be applied by setting `normalize=True`.
     """
-    import matplotlib
-
-    matplotlib.use("Agg")
-    import matplotlib.pyplot as plt
-
-    cmap = plt.cm.Oranges
+    cmap = cm.Oranges
 
     if normalize:
         cm = cm.astype(np.float32) / cm.sum(axis=1)[:, np.newaxis]
