@@ -1,34 +1,34 @@
 from __future__ import division
 from __future__ import print_function
+
+import inspect
 import math
 import os
-import inspect
-import numpy as np
-from collections import *
-from functools import partial
 import uuid
-from copy import copy, deepcopy
+from collections import *
 from collections import deque
+from copy import copy, deepcopy
+from functools import partial
+from itertools import repeat
+
+import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
-from torch._six import container_abcs
-from itertools import repeat
-
-
 
 from ..backend.common import *
-from ..backend.pytorch_backend import to_numpy,to_tensor,Layer,Sequential,Input,summary
-from ..layers.pytorch_layers import *
-from ..layers.pytorch_activations import  get_activation,Identity,Relu
-from ..layers.pytorch_normalizations import get_normalization,BatchNorm2d
-from ..layers.pytorch_blocks import *
-from ..layers.pytorch_pooling import *
-from ..optims.pytorch_trainer import *
+from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, summary
 from ..data.image_common import *
 from ..data.utils import download_model_from_google_drive
+from ..layers.pytorch_activations import get_activation, Identity, Relu
+from ..layers.pytorch_blocks import *
+from ..layers.pytorch_layers import *
+from ..layers.pytorch_normalizations import get_normalization, BatchNorm2d
+from ..layers.pytorch_pooling import *
+from ..optims.pytorch_trainer import *
 
 __all__ = ['DenseNet','DenseNet121','DenseNet161','DenseNet169','DenseNet201','DenseNetFcn']
 
