@@ -1,4 +1,4 @@
-from __future__ import absolute_import
+rom __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 import math
@@ -8,6 +8,19 @@ import cntk as C
 import cntk.ops
 from ..backend.common import get_session,addindent,get_time_suffix,get_function,get_class,format_time,get_terminal_size,snake2camel,camel2snake
 
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+
+import collections
+import itertools
+import math
+
+import cntk as C
+import cntk.ops
+
+from ..backend.common import get_session, addindent, get_time_suffix, get_function, get_class, format_time, \
+    get_terminal_size, snake2camel, camel2snake
 
 __all__ = ['accuracy','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
 
