@@ -1,25 +1,28 @@
+import random
+from math import *
+
+import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torchvision
-import random
 from torch.autograd import Variable
-import numpy as np
-from math import *
-from torch.nn.modules.loss import _Loss, _WeightedLoss
-from torch.nn import init
 from torch.nn import _reduction as _Reduction
-from ..data.image_common import *
-from ..backend.common import *
-from ..layers.pytorch_activations import sigmoid
-from ..backend.pytorch_backend import *
-from ..backend.pytorch_ops import *
+from torch.nn import init
+from torch.nn.modules.loss import _Loss, _WeightedLoss
 from torchvision import transforms
 from torchvision.transforms import functional as tvf
+
+from ..backend.common import *
+from ..backend.pytorch_backend import *
+from ..backend.pytorch_ops import *
+from ..data.image_common import *
+from ..layers.pytorch_activations import sigmoid
+
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
 _session = get_session()
 
-__all__ = ['MSELoss', 'CrossEntropyLoss', 'NullLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'SmoothL1Loss', 'L2Loss','CosineSimilarityLoss',
+__all__ = ['MSELoss', 'CrossEntropyLoss', 'NllLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'SmoothL1Loss', 'L2Loss', 'CosineSimilarityLoss',
            'ExponentialLoss','ItakuraSaitoLoss', 'make_onehot', 'MS_SSIMLoss', 'CrossEntropyLabelSmooth', 'mixup_criterion', 'DiceLoss',
            'IouLoss', 'focal_loss_with_logits', 'FocalLoss', 'SoftIoULoss', 'CenterLoss', 'TripletLoss',
            'LovaszSoftmax', 'PerceptionLoss', 'EdgeLoss', 'TransformInvariantLoss', 'get_loss']
@@ -42,7 +45,7 @@ class CrossEntropyLoss(_Loss):
                                reduction=self.reduction)
 
 
-class NullLoss(_Loss):
+class NllLoss(_Loss):
     def __init__(self, weight=None, with_logits=True, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
         super().__init__(reduction=reduction)
         self.name = name
@@ -63,7 +66,7 @@ class NullLoss(_Loss):
     #     output.shape)>2 and len(target.shape)==len(output.shape)-1:  #         return super().forward(output.view(
     #     -1,output.size(1)), target.long().view(-1))  #     elif len(output.shape)>2 and len(target.shape)==len(
     #     output.shape):  #         return super().forward(output.view(-1,output.size(1)), argmax(target,1).view(-1))
-
+NullLoss=NllLoss
 
 class BCELoss(_Loss):
     def __init__(self, weight=None, reduction='mean', with_logit=False, name='BCELoss'):
