@@ -1,25 +1,26 @@
+import itertools as it
+import math
 import os
 import sys
 import time
-import itertools as it
-from shutil import copyfile
 import uuid
-import math
+from collections import OrderedDict, defaultdict
+from functools import partial
+from shutil import copyfile
+
+import numpy as np
 import torch
 import torch.nn as nn
-import torch.optim as optim
-
-from torch.optim.optimizer import  Optimizer
 import torch.nn.functional as F
-from torch.autograd import Variable
+import torch.optim as optim
 import torch.utils.hooks as hooks
-from collections import OrderedDict,defaultdict
-from functools import partial
-import numpy as np
-from ..backend.common import get_session,addindent,get_time_suffix,get_class,format_time,get_terminal_size,snake2camel,camel2snake
-from ..backend.pytorch_backend import *
-from .trainers import OptimizerBase
+from torch.autograd import Variable
+from torch.optim.optimizer import Optimizer
 
+from .trainers import OptimizerBase
+from ..backend.common import get_session, addindent, get_time_suffix, get_class, format_time, get_terminal_size, \
+    snake2camel, camel2snake
+from ..backend.pytorch_backend import *
 
 __all__ = ['Adam','SGD','LBFGS','Adadelta','Adagrad','RMSprop','RAdam','PlainRAdam','AdamW','Lookahead','Ranger','get_optimizer']
 
