@@ -1,13 +1,11 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
-
 import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 from ..backend.common import get_session,addindent,get_time_suffix,get_class,format_time,get_terminal_size,snake2camel,camel2snake,get_function
+from ..backend.pytorch_ops import *
+from ..backend.pytorch_backend import to_tensor,to_numpy
+
 
 
 _session = get_session()
@@ -17,22 +15,17 @@ _epsilon = _session.epsilon
 __all__ = ['l1_reg','l2_reg','orth_reg','get_reg','total_variation_norm_reg']
 
 
-_device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-_session=get_session()
-_epsilon=_session.epsilon
-
-
 def l1_reg(model:nn.Module,reg_weight=1e-6):
     #with torch.enable_grad():
-    loss =torch.tensor(0,dtype=torch.float,requires_grad=True).to(_device)
+    loss =0.0
     for name, param in model.named_parameters():
         if 'bias' not in name:
-            loss = loss + (reg_weight * torch.sum(torch.abs(param)))
+            loss = loss + (reg_weight * torch.sum(abs(param)))
         return loss
 
 
 def l2_reg(model:nn.Module,reg_weight=1e-6):
-    loss =0
+    loss =0.0
     for name, param in model.named_parameters():
         if 'bias' not in name:
             loss=loss+ reg_weight *torch.norm(param)
@@ -40,7 +33,7 @@ def l2_reg(model:nn.Module,reg_weight=1e-6):
 
 
 def orth_reg(model:nn.Module,reg_weight=1e-6):
-    loss =torch.tensor(0,dtype=torch.float,requires_grad=True).to(_device)
+    loss =0.0
     for name, param in model.named_parameters():
         if 'bias' not in name:
             param_flat = param.view(param.shape[0], -1)
@@ -52,7 +45,7 @@ def orth_reg(model:nn.Module,reg_weight=1e-6):
 def total_variation_norm_reg(output:torch.Tensor,reg_weight=1):
     total_variation_norm_reg.reg_weight=reg_weight
     assert len(output.size())==4
-    loss = reg_weight * (torch.sum(torch.abs(output[:, :, :, :-1] - output[:, :, :, 1:])) + torch.sum(torch.abs(output[:, :, :-1, :] - output[:, :, 1:, :])))/2.0
+    loss = reg_weight * (torch.sum(abs(output[:, :, :, :-1] - output[:, :, :, 1:])) + torch.sum(abs(output[:, :, :-1, :] - output[:, :, 1:, :])))/2.0
     return loss
 
 
