@@ -1,28 +1,30 @@
-import os
 import copy
 import inspect
-import time
+import os
 import random
-import sys
+import shutil
 import string
+import sys
+import time
 import uuid
+from functools import partial
+
+import numpy as np
 import torch
 import torch.nn as nn
-import shutil
-from functools import partial
-import numpy as np
+
+from .pytorch_constraints import get_constraint
+from .pytorch_losses import get_loss
+from .pytorch_metrics import get_metric
+from .pytorch_optimizers import get_optimizer
+from .pytorch_regularizers import get_reg
+from .trainers import ModelBase, OptimizerBase, progress_bar
 from ..backend.common import *
 from ..backend.pytorch_backend import *
 from ..backend.pytorch_ops import *
-from .pytorch_optimizers import get_optimizer
-from .pytorch_losses import get_loss
-from .pytorch_metrics import get_metric
-from .pytorch_regularizers import get_reg
-from .pytorch_constraints import get_constraint
-from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 from ..callbacks.lr_schedulers import get_lr_scheduler
-from .trainers import ModelBase, OptimizerBase, progress_bar
 from ..data.image_common import *
+from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 
 __all__ = ['TrainingItem', 'Model', 'ImageClassificationModel', 'ImageDetectionModel', 'ImageGenerationModel',
            'ImageSegmentationModel','FaceRecognitionModel']
@@ -174,7 +176,7 @@ class Model(ModelBase):
     def train(self):
         if self._model is not None and  isinstance(self._model, torch.Tensor):
             pass
-        elif self._model is not None and self._model.built:
+        elif self._model is not None and isinstance(self._model,Layer) and self._model.built:
             self._model.train()
         else:
             raise ValueError('There is no built model ,nothing to learn')
@@ -182,7 +184,7 @@ class Model(ModelBase):
     def eval(self):
         if self._model is not None and  isinstance(self._model, torch.Tensor):
             pass
-        elif self._model is not None and self._model.built:
+        elif self._model is not None and isinstance(self._model,Layer)and self._model.built:
             self._model.eval()
         else:
             raise ValueError('There is no built model ,nothing to evaluate')
@@ -470,17 +472,10 @@ class Model(ModelBase):
                     self.targets[self.targets.key_list[k]] = list(out[k].size())[1:] if len(out[k].size()) > 1 else []
 
     def do_on_training_start(self):
-        if self._model is not None and  isinstance(self._model, torch.Tensor):
-            pass
-        else:
-            self._model.train()
-            self._model.zero_grad()
+        self.train()
 
     def do_on_training_end(self):
-        if self._model is not None and  isinstance(self._model, torch.Tensor):
-            pass
-        else:
-            self._model.eval()
+        self.eval()
 
     def do_on_epoch_start(self):
         # if self.training_context['current_epoch'] < self.warmup:
@@ -743,7 +738,7 @@ class Model(ModelBase):
 
     def load_model(self, file_path):
         print('Loading pretrained model from {}'.format(file_path))
-        pretrained_dict = torch.load(file_path, map_location=lambda storage, loc: storage)
+        pretrained_dict = torch.load(file_path,  map_location=torch.device(get_device()))
 
         if "state_dict" in pretrained_dict.keys():
             pretrained_dict = pretrained_dict['state_dict']
@@ -759,7 +754,7 @@ class Model(ModelBase):
 
     def summary(self):
         # self.rebinding_input_output(self._model.input_shape)
-        summary(self._model, tuple(self._model.input_shape))
+        summary(self._model, self.inputs.value_list)
 
     def predict(self,input):
         raise NotImplementedError
