@@ -1,18 +1,20 @@
-import os
-import sys
-import inspect
-import uuid
-import time
 import copy
+import inspect
 import json
+import os
 import shutil
+import sys
+import time
+import uuid
 from functools import partial
+
 import numpy as np
+
 from ..backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session, \
-    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict,split_path,sanitize_path
-from ..data.data_provider import *
+    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict, split_path, sanitize_path
 from ..callbacks import *
 from ..callbacks.visualization_callbacks import *
+from ..data.data_provider import *
 from ..misc.ipython_utils import *
 from ..misc.visualization_utils import tile_rgb_images, loss_metric_curve
 
@@ -221,7 +223,7 @@ class ModelBase(object):
                                  'loss_history': None, 'metric_history': None, 'base_lr': self.base_lr,  # current loss
                                  'current_lr': None,  # current loss
                                  'save_path': None, 'is_collect_data': True, 'callbacks': self.callbacks,
-                                 'stop_update': False, 'retain_graph': False}
+                                 'stop_update': 0, 'retain_graph': False}
         self.training_context['losses']['total_losses'] = []
 
         self._initial_graph(inputs, output, input_shape)
@@ -469,7 +471,7 @@ class ModelBase(object):
                                  'best_model': None,  # current model
                                  'loss_history': None, 'metric_history': None, 'base_lr': self.base_lr,  # current loss
                                  'current_lr': None,  # current loss
-                                 'save_path': None, 'callbacks': self.callbacks, 'stop_update': False,
+                                 'save_path': None, 'callbacks': self.callbacks, 'stop_update': 0,
                                  'retain_graph': False}
 
     def adjust_learning_rate(self, lr):
@@ -527,7 +529,7 @@ class ModelBase(object):
         pass
 
     def do_post_gradient_update(self):
-        self.training_context['tmp_losses'].append(self.training_context['current_loss'].item())
+        self.training_context['tmp_losses'].append(to_numpy(self.training_context['current_loss']).mean())
         if self.training_context['is_collect_data'] == True:
             self.training_context['losses']['total_losses'].append(
                 float(to_numpy(self.training_context['tmp_losses']).mean()))
@@ -747,7 +749,7 @@ class ModelBase(object):
                     if k + '_Loss' not in self.training_context['losses']:
                         self.training_context['losses'][k + '_Loss'] = []
 
-                    this_loss = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else torch.tensor(0)
+                    this_loss = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else to_tensor(0)
                     overall_loss=None
                     if isinstance(this_loss, tuple):
                         overall_loss = this_loss[0]
@@ -799,7 +801,10 @@ class ModelBase(object):
                         v(self._model)
 
                 if log_weights and is_collect_data:
-                    self.log_weight()
+                    if isinstance(self._model,Layer):
+                        self.log_weight(weghts=self._model.weights)
+                    elif is_tensor(self._model):
+                        self.log_weight(weghts=self._model)
 
                 if test_data is not None and len(test_data) > 0 and  self.training_context['stop_update']<1 :
                     output = try_map_args_and_call(self._model, test_data, self.training_context['data_feed'])
@@ -1064,13 +1069,13 @@ class TrainingPlan(object):
         else:
             if len(name) > 0:
                 training_item.name = name
-                if isinstance(training_item.model,nn.Module):
-                    training_item.model.name=name
+                if isinstance(training_item.model,Layer):
+                    training_item.model._name=name
                 self.training_names[n] = name
             else:
                 training_item.name = 'model {0}'.format(n)
-                if isinstance(training_item.model, nn.Module):
-                    training_item.model.name ='model {0}'.format(n)
+                if isinstance(training_item.model, Layer):
+                    training_item.model._name ='model {0}'.format(n)
                 self.training_names[n] = 'model {0}'.format(n)
         self.training_items[n] = training_item
         self.training_items[n].start_epoch = start_epoch
@@ -1284,7 +1289,7 @@ class TrainingPlan(object):
                             trainitem.training_context['model_name']=trainitem_name
                             if epoch<trainitem.start_epoch:
                                 trainitem.training_context['stop_update']=1
-                            trainitem.train_model(train_data,test_data, epoch, mbs if only_steps==False else  num_batches, self.num_epochs if only_steps==False else  1,
+                            trainitem.train_model(train_data,test_data, epoch if only_steps==False else 0,mbs if only_steps==False else  num_batches, self.num_epochs if only_steps==False else  1,
                                                   len(data_loader.batch_sampler) if only_steps==False else max_batches,
                                                   is_collect_data=mbs % collect_data_inteval == 0,
                                                   is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
@@ -1305,9 +1310,12 @@ class TrainingPlan(object):
                                 trainitem.save_model()
 
 
-                        if num_batches == max_batches - 1:
+                        if only_steps==True and num_batches >= max_batches - 1:
                             for k, trainitem in self.training_items.items():
-                                trainitem.save_model()
+                                try:
+                                     trainitem.save_model()
+                                except Exception as e:
+                                    print(e)
                             return True
 
                         if only_steps==False and (mbs + 1) % len(data_loader.batch_sampler) == 0:
@@ -1347,111 +1355,7 @@ class TrainingPlan(object):
 
     def only_steps(self, num_steps, collect_data_inteval=1, keep_weights_history=False, keep_gradient_history=False):
         return self.start_now(collect_data_inteval=collect_data_inteval, is_resume=False,only_steps=True,max_batches=num_steps,keep_weights_history=keep_weights_history, keep_gradient_history=keep_gradient_history)
-        # self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-','') if self.execution_id is None else self.execution_id
-        #
-        # # update callback
-        #
-        # for item in self.training_items.values():
-        #     for callback in self.callbacks:
-        #         if callback not in item.callbacks:
-        #             # private callback
-        #             if callback.is_shared == False:
-        #                 item.with_callbacks(copy.deepcopy(callback))
-        #             else:
-        #                 # shared callback
-        #                 item.with_callbacks(callback)
-        # # shared callbacks will access training plan dict instead of training_context
-        # for callback in self.callbacks:
-        #     if callback.is_shared == True:
-        #         callback.on_training_start(self.__dict__)
-        #
-        # data_loader = self._dataloaders.value_list[0]
-        # data_loader.minibatch_size = self.minibatch_size
-        # # generate data feed
-        #
-        # self.generate_datafeed(data_loader)
-        #
-        # try:
-        #
-        #     for mbs, return_data in enumerate(data_loader):
-        #         if mbs < num_steps:
-        #             iter_data = OrderedDict()
-        #             for i in range(len(data_loader.signature.key_list)):
-        #                 name = data_loader.signature.key_list[i]
-        #                 iter_data[name] = return_data[i]
-        #
-        #             # check weather need out-of-sample evaluation
-        #             need_out_sample_evaluation = False
-        #             if self.out_sample_evaluation_on_epoch_end == True and mbs == len(data_loader.batch_sampler) - 1:
-        #                 need_out_sample_evaluation = True
-        #             if self.out_sample_evaluation_unit == 'batch' and mbs > 0 and mbs % self.out_sample_evaluation_frequency == 0:
-        #                 need_out_sample_evaluation = True
-        #
-        #             iter_testdata = None
-        #             if isinstance(data_loader,
-        #                           DataProviderV2) and data_loader.testdata is not None and need_out_sample_evaluation:
-        #                 return_test = data_loader.next_test()
-        #                 if return_test is not None:
-        #                     iter_testdata = OrderedDict()
-        #                     for i in range(len(data_loader.signature.key_list)):
-        #                         name = data_loader.signature.key_list[i]
-        #                         iter_testdata[name] = return_test[i]
-        #
-        #             # input, target = Variable(input).to(self.device), Variable(target).to(self.device)
-        #
-        #             for trainitem_name, trainitem in zip(self.training_names.value_list,
-        #                                                  self.training_items.value_list):
-        #                 train_data = OrderedDict()
-        #                 test_data = None if iter_testdata is None else OrderedDict()
-        #                 for k, v in iter_data.items():
-        #                     train_data[k] = v.copy()
-        #
-        #                 if iter_testdata is not None:
-        #                     for k, v in iter_testdata.items():
-        #                         test_data[k] = v.copy()
-        #
-        #                 trainitem.training_context['model_name'] = trainitem_name
-        #                 trainitem.train_model(train_data,test_data, 0, mbs, 1, num_steps,
-        #                                       is_collect_data=mbs % collect_data_inteval == 0,
-        #                                       is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
-        #                                       is_print_epoch_progress=False,
-        #                                       log_gradients=keep_gradient_history and mbs % collect_data_inteval == 0,
-        #                                       log_weights=keep_weights_history and mbs % collect_data_inteval == 0,
-        #                                       accumulate_grads=False)
-        #             if len(self.training_items.items()) > 2 and self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0:
-        #                 print('\n')
-        #
-        #         else:
-        #             break
-        #     # if self.print_progress_on_epoch_end:
-        #     #     progress_bar(num_steps, num_steps, 'Loss: {0:<8.3f}| {1} | learning rate: {2:<10.4e}'.format(
-        #     #         np.array(self.training_items[0].batch_loss_history['total_losses']).mean(),
-        #     #         ','.join(['{0}: {1:<8.3%}'.format(snake2camel(k), np.array(v).mean()) for k,
-        #     #         v in self.training_items[0].batch_metric_history.items()]),
-        #     #         optimizer.param_groups[0]['lr']))
-        #
-        #     batch_loss_history = [trainitem.batch_loss_history for k, trainitem in self.training_items.items()]
-        #     batch_metric_history = [trainitem.batch_metric_history for k, trainitem in self.training_items.items()]
-        #     loss_metric_curve(batch_loss_history, batch_metric_history, legend=self.training_names.value_list,
-        #                       sample_collected=self.training_items[0].sample_collect_history, max_iteration=num_steps,
-        #                       calculate_base='batch', save_path=self.tile_image_save_path,
-        #                       imshow=self.tile_image_imshow)
-        #
-        #
-        # except StopIteration:
-        #     for k, trainitem in self.training_items.items():
-        #         trainitem.do_on_training_end()
-        # except ValueError as ve:
-        #     for k, trainitem in self.training_items.items():
-        #         trainitem.do_on_training_end()
-        #     print(ve)
-        #     PrintException()
-        # except Exception as e:
-        #     for k, trainitem in self.training_items.items():
-        #         trainitem.do_on_training_end()
-        #     print(e)
-        #     PrintException()
-        #
+
 
 last_time = time.time()
 begin_time = last_time
